/*************************************************************************
* (C) 2008 AITNET ltd - Sofia/Bulgaria - <misho@aitbg.com>
*  by Michael Pounov <misho@openbsd-bg.org>
*
* $Author: misho $
* $Id: aitcfg.h,v 1.11 2013/05/30 09:12:27 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#ifndef __AITCFG_H
#define __AITCFG_H


#include <pthread.h>
#include <elwix.h>


struct tagCfg {
	ait_val_t		cfg_sec;
	ait_val_t		cfg_attr;
	ait_val_t		cfg_val;

	SLIST_ENTRY(tagCfg)	cfg_next;
	RB_ENTRY(tagCfg)	cfg_node;
};
typedef struct tagRC {
	pthread_mutex_t		rc_mtx;

	struct tagCfg		*slh_first;
	struct tagCfg		*rbh_root;
} cfg_root_t;
#define CFG_RC_LOCK(x)		pthread_mutex_lock(&(x)->rc_mtx)
#define CFG_RC_UNLOCK(x)	pthread_mutex_unlock(&(x)->rc_mtx)


// cfg_GetErrno() Get error code of last operation
int cfg_GetErrno();
// cfg_GetError() Get error text of last operation
const char *cfg_GetError();


/*
 * Macros for config library.
 */
#define CFG_ISEMPTY(x)		RB_EMPTY((x))


/*
 * cfgInitConfig() - Init config root
 *
 * @cfg = Config root
 * return: -1 error or 0 ok
 */
int cfgInitConfig(cfg_root_t * __restrict cfg);
/*
 * cfgLoadConfig() - Load config from file
 *
 * @cfgName = Config filename
 * @cfg = Config root
 * return: -1 error or 0 ok
 */
int cfgLoadConfig(const char *cfgName, cfg_root_t * __restrict cfg);
/*
 * cfgUnloadConfig() - Unload config from memory and destroy resources
 *
 * @cfg = Config root
 * return: none
 */
void cfgUnloadConfig(cfg_root_t * __restrict cfg);
/*
 * cfgClearConfig() - Clear config and free resources
 *
 * @cfg = Config root
 * return: none
 */
void cfgClearConfig(cfg_root_t * __restrict cfg);
/*
 * cfgCreateConfig() - Create config file from memory
 *
 * @csConfigName = New config filename
 * @cfg = Config root
 * @whitespace = Additional whitespace characters to file
 * return: -1 error or 0 ok
 */
int cfgCreateConfig(const char *csConfigName, cfg_root_t * __restrict cfg, 
		int whitespace);

/*
 * cfgReadLines() - Read custom lines and add new item at config root
 *
 * @f = File resource
 * @delim = Custom delimiter, if =NULL default is '='
 * @end = Custom user end of file, if =NULL default is EOF
 * @cfg = Config root
 * return: -1 error or 0 ok
 */
int cfgReadLines(FILE *f, const char *delim, const char *end, 
		cfg_root_t * __restrict cfg);
/*
 * cfgWriteLines() - Write custom lines and export data to variable
 *
 * @f = File resource
 * @delim = Custom delimiter, if =NULL default is '='
 * @eol = End of line string, if =NULL default is "\n"
 * @section = Export only section, if =NULL default is all
 * @cfg = Config root
 * return: =NULL error or !=NULL exported data, must be free after use with ait_freeVar()
 */
ait_val_t *cfgWriteLines(FILE *f, const char *delim, const char *eol, 
		const char *section, cfg_root_t * __restrict cfg);
/*
 * cfgReadConfig() - Read file and add new item at config root
 *
 * @f = File resource
 * @cfg = Config root
 * return: -1 error or 0 ok
 */
int cfgReadConfig(FILE *f, cfg_root_t * __restrict cfg);
/*
 * cfgWriteConfig() - Write config from memory
 *
 * @f = File handle
 * @cfg = Config root
 * @whitespace = Additional whitespace characters to file
 * return: -1 error or 0 ok
 */
int cfgWriteConfig(FILE *f, cfg_root_t * __restrict cfg, int whitespace);
/*
 * cfgConcatConfig() - Concat two configs into one
 *
 * @cfg = Config root
 * @add_cfg = Concated config will be destroy after merge
 * return: -1 error or 0 ok
 */
int cfgConcatConfig(cfg_root_t * __restrict cfg, cfg_root_t * __restrict add_cfg);
/*
 * cfgMergeConfig() - Marge two list in one cfg and destroy add_cfg
 *
 * @cfg = Config root of main list
 * @add_cfg = Merged config will be destroy after merge
 * return: -1 error or 0 ok
 */
int cfgMergeConfig(cfg_root_t * __restrict cfg, cfg_root_t * __restrict add_cfg);

/*
 * cfg_findAttribute() - Find attribute position in config file
 *
 * @cfg = Config root
 * @csSec = Config section //[{csSec}]
 * @csAttr = Config attribute //{csAttr} = ...
 * return: 0 not found item, -1 error or >0 position in list
 */
int cfg_findAttribute(cfg_root_t * __restrict cfg, 
		const char *csSec, const char *csAttr);
/*
 * cfg_getAttribute() - Get item from config and return value from it
 *
 * @cfg = Config root
 * @csSec = Config section //[{csSec}], if NULL unset in *default* section
 * @csAttr = Config attribute //{csAttr} = ..., if NULL unset as *any* attribute
 * return: NULL item not found or null parameters, !=NULL value const string
 */
const char *cfg_getAttribute(cfg_root_t * __restrict cfg, 
		const char *csSec, const char *csAttr);
/*
 * cfg_setAttribute() - Set item in config or adding new item if not exists
 *
 * @cfg = Config root
 * @csSec = Config section //[{csSec}], if NULL set in *default* section
 * @csAttr = Config attribute //{csAttr} = ...
 * @csVal = Config value //... = {csVal} to setup
 * return: 0 nothing changed, -1 error, 1 found and updated item or 2 added new item
 */
int cfg_setAttribute(cfg_root_t * __restrict cfg, const char *csSec, 
		const char *csAttr, const char *csVal);
/*
 * cfg_unsetAttribute() - Unset item from config and free resources
 *
 * @cfg = Config root
 * @csSec = Config section //[{csSec}], if NULL unset in *default* section
 * @csAttr = Config attribute //{csAttr} = ...
 * return: 0 item not found, -1 error or 1 removed item
 */
int cfg_unsetAttribute(cfg_root_t * __restrict cfg, const char *csSec, 
		const char *csAttr);

/*
 * cfg_loadAttribute() - Get guarded attribute, if not found item return *default value*
 *
 * @cfg = Config root
 * @csSec = Config section //[{csSec}], if NULL unset in *default* section
 * @csAttr = Config attribute //{csAttr} = ...
 * @val = Return buffer for item Value //... = {val}
 * @csDefValue = *Default Value* for return in //{val}, if not found item in config
 * return: 0 item not found, -1 error or >0 number of copied bytes in //{val}
 */
int cfg_loadAttribute(cfg_root_t * __restrict cfg, const char *csSec, 
		const char *csAttr, ait_val_t * __restrict val, const char *csDefValue);


#endif
