/*************************************************************************
* (C) 2010 AITNET ltd - Sofia/Bulgaria - <misho@aitbg.com>
*  by Michael Pounov <misho@openbsd-bg.org>
*
* $Author: misho $
* $Id: aitrpc.h,v 1.22 2013/12/15 23:01:02 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#ifndef __AITRPC_H
#define __AITRPC_H


#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/param.h>
#if !defined(__NetBSD__)
#include <sys/limits.h>
#endif
#include <sys/socket.h>
#include <sys/queue.h>
#include <elwix.h>
#include <aitsched.h>


#define RPC_VERSION		7
#define RPC_DEFPORT		2611

/* RPC call request flags */

#define RPC_REPLY		0x0
#define RPC_NOREPLY		0x1

/* RPC builtin registed calls */

#define CALL_TAG_MAX		65535

#define CALL_SRVPING		65534

#define CALL_SRVSHUTDOWN	65533
#define CALL_SRVCLIENTS		65532
#define CALL_SRVCALLS		65531
#define CALL_SRVSESSIONS	65530

#define CALL_BLOBSHUTDOWN	65529
#define CALL_BLOBCLIENTS	65528
#define CALL_BLOBVARS		65527

/* RPC signals */
#define SIGFBLOB		54

/* RPC types */

typedef enum {
	ok, error, no, 				/* for blob reply */
	get, set, unset				/* for blob request */
} blob_cmd_type_t;


#define RPC_CALLBACK_CHECK_INPUT(x)	do { \
						assert((x)); \
						if (!(x)) { \
							rpc_SetErr(EINVAL, \
									"Invalid callback parameters ..."); \
							return -1; \
						} \
					} while (0)


/* RPC session identification */

typedef struct {
#if BYTE_ORDER == LITTLE_ENDIAN
	uint16_t	sess_instance:8;
	uint16_t	sess_version:8;
#endif
#if BYTE_ORDER == BIG_ENDIAN
	uint16_t	sess_version:8;
	uint16_t	sess_instance:8;
#endif
} __packed rpc_sess_t;	/* size == 2 bytes */


/* Server managment RPC functions ... */

/* Network RPC packet - Client request */

struct tagRPCCall {
	rpc_sess_t	call_session;

	uint32_t	call_len;
	uint16_t	call_crc;

	union {
		struct {
			uint64_t	flags;
		}	call_req;
		struct {
			int32_t		ret;
			int32_t		eno;
		}	call_rep;
	};

	uint16_t	call_tag;
	uint16_t	call_argc;
	ait_val_t	call_argv[0];
} __packed;			/* size == 20 bytes */
#define RPC_CHK_NOREPLY(x)	(ntohl((u_long) (x)->call_req.flags) & RPC_NOREPLY)
#define RPC_SET_ERRNO(x, _v)	((x)->call_rep.eno = htonl((_v)))

/* Network BLOB packet - Header */

struct tagBLOBHdr {
	rpc_sess_t	hdr_session;
	uint8_t		hdr_cmd;
	uint32_t	hdr_var;
	uint32_t	hdr_len;
	uint32_t	hdr_ret;
	uint8_t		hdr_pad;
} __packed;			/* size == 16 bytes */

/* Network RPC client & server elements */

/* RPC function registration element! */
typedef struct tagRPCFunc {
	ait_val_t		func_name;

	void			*func_parent;

	SLIST_ENTRY(tagRPCFunc)	func_next;
	AVL_ENTRY(tagRPCFunc)	func_node;
} rpc_func_t;
#define RPC_FUNC_SERVER(x)	((rpc_srv_t*) (x)->func_parent)

/* Tree root node */
typedef struct tagRPCFuncs {
	pthread_mutex_t		mtx;

	struct tagRPCFunc	*slh_first;
	struct tagRPCFunc	*avlh_root;
} rpc_funcs_t;
#define RPC_FUNCS_LOCK(x)	pthread_mutex_lock(&(x)->mtx)
#define RPC_FUNCS_UNLOCK(x)	pthread_mutex_unlock(&(x)->mtx)
#define RPC_FUNCS_ISEMPTY(x)	AVL_EMPTY((x))


/* BLOB register element */
typedef struct tagBLOB {
	uint32_t		blob_var;	/* BLOB id */

	size_t			blob_len;	/* size of allocated BLOB data */
	void			*blob_data;	/* mapped BLOB data */

	TAILQ_ENTRY(tagBLOB)	blob_node;
} rpc_blob_t;


typedef struct {
	int		cli_id;		/* slot id */
	int		cli_sock;	/* socket fd */
	sockaddr_t	cli_sa;		/* host address */
	ait_val_t	cli_buf;	/* network buffer */

	array_t		*cli_vars;	/* function return variables */

	void		*cli_parent;	/* pointer to parent rpc_srv_t for server or to rpc_sess_t for client */
} rpc_cli_t;
#define RPC_RETVARS(x)		((x)->cli_vars)
#define RPC_SRV_SERVER(x)	((rpc_srv_t*) (x)->cli_parent)
#define RPC_CLI_SESSION(x)	((rpc_sess_t*) (x)->cli_parent)

typedef struct {
	rpc_sess_t			srv_session;	/* RPC session registration info */
	int				srv_netbuf;	/* size of network buffer */
	int				srv_proto;	/* Server protocol */

	pthread_t			srv_tid;	/* RPC exec pthread */
	sched_root_task_t		*srv_root;	/* RPC server scheduler */
	intptr_t			srv_kill;	/* Scheduler condition variable */

	rpc_cli_t			srv_server;	/* RPC server socket */
	array_t				*srv_clients;	/* connected rpc client sockets */

	rpc_funcs_t			srv_funcs;	/* RPC functions */

	struct {
		pthread_t			tid;		/* BLOB exec pthread */
		sched_root_task_t		*root;		/* BLOB server scheduler */
		intptr_t			kill;		/* BLOB server state: ==0 disable | !=0 enable */

		ait_val_t			dir;		/* BLOB states directory */

		rpc_cli_t			server;		/* BLOB server socket */
		array_t				*clients;	/* connected blob client sockets */

		TAILQ_HEAD(, tagBLOB)		blobs;		/* registered blob variables list */
	} 				srv_blob;
} rpc_srv_t;


/* 
 * (*rpc_callback_t)() - Callback type definition for RPC call in server process
 *
 * @arg1 = RPC client
 * @arg2 = RPC packet header
 * @arg3 = input array with values from RPC call execution request
 * return: -1 error or >-1 success execution
 */
typedef int (*rpc_callback_t)(rpc_cli_t *, struct tagRPCCall *, array_t *);


/* ----------------------------------------------------------------------- */

/* Error support functions */

// rpc_GetErrno() Get error code of last operation
int rpc_GetErrno();
// rpc_GetError() Get error text of last operation
const char *rpc_GetError();
// rpc_SetErr() Set error to variables for internal use!!!
void rpc_SetErr(int eno, char *estr, ...);



/*
 * rpc_Read() - RPC read operation
 *
 * @sock = socket
 * @type = type of socket
 * @flags = receive flags
 * @sa = check client address, if you use udp protocol
 * @buf = buffer
 * @blen = buffer length
 * return: -1 error, 0 EOF or or >0 readed bytes into buffer
 */
ssize_t rpc_Read(int sock, int type, int flags, sockaddr_t * __restrict sa, 
		unsigned char * __restrict buf, size_t blen);
/*
 * rpc_Write() - RPC write operation
 *
 * @sock = socket
 * @type = type of socket
 * @flags = send flags
 * @sa = send to client address, if you use udp protocol
 * @buf = buffer
 * @blen = buffer length
 * return: -1 error, 0 EOF or >0 written bytes into buffer
 */
ssize_t rpc_Write(int sock, int type, int flags, sockaddr_t * __restrict sa, 
		unsigned char * __restrict buf, size_t blen);

/*
 * rpc_chkPktSession() - Check RPC session
 *
 * @p = packet session
 * @s = active session
 * return: -1, 1, 2, 3 are errors or 0 ok
 */
int rpc_chkPktSession(rpc_sess_t *p, rpc_sess_t *s);
/*
 * rpc_addPktSession() - Prepare session into network format
 *
 * @p = packet session
 * @s = host session
 * return: -1 error or 0 ok
 */
int rpc_addPktSession(rpc_sess_t *p, rpc_sess_t *s);
/*
 * rpc_register_srvPing() - Register ping service function
 *
 * @srv = RPC server instance
 * return: -1 error or 0 ok
 */
int rpc_register_srvPing(rpc_srv_t * __restrict srv);
/*
 * rpc_register_srvServices() - Register internal service functions
 *
 * @srv = RPC server instance
 * return: -1 error or 0 ok
 */
int rpc_register_srvServices(rpc_srv_t * __restrict srv);
/*
 * rpc_register_blobServices() - Register internal service functions
 *
 * @srv = RPC server instance
 * return: -1 error or 0 ok
 */
int rpc_register_blobServices(rpc_srv_t * __restrict srv);


/* RPC Server side functions */

/*
 * rpc_srv_initServer() - Init & create RPC Server
 *
 * @InstID = Instance for authentication & recognition
 * @concurentClients = Concurent clients at same time to this server
 * @netBuf = Network buffer length (min:512 bytes), if =0 == BUFSIZ (also meaning max RPC packet)
 * @csHost = Host name or address for bind server, if NULL any address
 * @Port = Port for bind server, if Port == 0 default port is selected
 * @proto = Protocol, if == 0 choose SOCK_STREAM
 * return: NULL == error or !=NULL bind and created RPC server instance
 */
rpc_srv_t *rpc_srv_initServer(unsigned char InstID, int concurentClients, int netBuf, 
		const char *csHost, unsigned short Port, int proto);
/*
 * rpc_srv_endServer() - Destroy RPC server, close all opened sockets and free resources
 *
 * @psrv = RPC Server instance
 * return: none
 */
void rpc_srv_endServer(rpc_srv_t ** __restrict psrv);
/*
 * rpc_srv_loopServer() - Execute Main server loop and wait for clients requests
 *
 * @srv = RPC Server instance
 * return: -1 error or 0 ok, infinite loop ...
 */
int rpc_srv_loopServer(rpc_srv_t * __restrict srv);
#define rpc_srv_execServer(_srv, _sync)	do { assert((_srv)); \
						if (!(_srv)->srv_kill) { \
							pthread_create(&(_srv)->srv_tid, NULL, (void*(*)(void*)) \
									rpc_srv_loopServer, (_srv)); \
							if ((_sync)) \
								pthread_join((_srv)->srv_tid, (void**) (_sync)); \
							else \
								pthread_detach((_srv)->srv_tid); \
						} } while (0)
#define rpc_srv_killServer(_srv)	(assert((_srv)), (_srv)->srv_blob.kill = 1, (_srv)->srv_kill = 1)

/*
 * rpc_srv_initBLOBServer() - Init & create BLOB Server
 *
 * @srv = RPC server instance
 * @Port = Port for bind server, if Port == 0 default port is selected
 * @diskDir = Disk place for BLOB file objects
 * return: -1 == error or 0 bind and created BLOB server instance
 */
int rpc_srv_initBLOBServer(rpc_srv_t * __restrict srv, unsigned short Port, const char *diskDir);
/*
 * rpc_srv_endBLOBServer() - Destroy BLOB server, close all opened sockets and free resources
 *
 * @srv = RPC Server instance
 * return: none
 */
void rpc_srv_endBLOBServer(rpc_srv_t * __restrict srv);
/*
 * rpc_srv_loopBLOB() - Execute Main BLOB server loop and wait for clients requests
 *
 * @srv = RPC Server instance
 * return: -1 error or 0 ok, infinite loop ...
 */
int rpc_srv_loopBLOBServer(rpc_srv_t * __restrict srv);
#define rpc_srv_execBLOBServer(_srv)	do { assert((_srv)); \
						if (!(_srv)->srv_kill && !(_srv)->srv_blob.kill) { \
							pthread_create(&(_srv)->srv_blob.tid, NULL, \
									(void*(*)(void*)) \
									rpc_srv_loopBLOBServer, (_srv)); \
							pthread_detach((_srv)->srv_blob.tid); \
						} \
					} while (0)

/*
 * rpc_srv_registerCall() - Register call to RPC server
 *
 * @srv = RPC Server instance
 * @tag = Function tag
 * @funcaddr = Function address
 * return: -1 error, 0 already registered tag or 1 register ok
 */
int rpc_srv_registerCall(rpc_srv_t * __restrict srv, unsigned short tag, void *funcaddr);
/*
 * rpc_srv_unregisterCall() - Unregister call from RPC server
 *
 * @srv = RPC Server instance
 * @tag = Function tag
 * return: -1 error, 0 not found call, 1 unregister ok
 */
int rpc_srv_unregisterCall(rpc_srv_t * __restrict srv, unsigned short tag);
/*
 * rpc_srv_getCall()  - Get registered call from RPC server
 *
 * @srv = RPC Server instance
 * @tag = tag for function
 * return: NULL not found call, !=NULL return call
 */
rpc_func_t *rpc_srv_getCall(rpc_srv_t * __restrict srv, uint16_t tag);
/*
 * rpc_srv_execCall() Execute registered call from RPC server
 *
 * @cli = RPC client
 * @rpc = IN RPC call structure
 * @funcname = Execute RPC function
 * @args = IN RPC calling arguments from RPC client
 * return: -1 error, !=-1 ok
 */
int rpc_srv_execCall(rpc_cli_t * __restrict cli, struct tagRPCCall * __restrict rpc, 
		ait_val_t funcname, array_t * __restrict args);


/*
 * rpc_srv_blobCreate() - Create and map blob to memory region and return object
 *
 * @srv = RPC Server instance
 * @len = BLOB length object
 * @tout = BLOB live timeout in seconds
 * return: NULL error or !=NULL allocated BLOB object
 */
rpc_blob_t *rpc_srv_blobCreate(rpc_srv_t * __restrict srv, int len, int tout);
/*
 * rpc_srv_blobMap() - Map blob to memory region 
 *
 * @srv = RPC Server instance
 * @blob = Map to this BLOB element
 * return: -1 error or 0 ok
 */
int rpc_srv_blobMap(rpc_srv_t * __restrict srv, rpc_blob_t * __restrict blob);
/*
 * rpc_srv_blobUnmap() - Unmap blob memory region 
 *
 * @blob = Mapped BLOB element
 * return: none
 */
void rpc_srv_blobUnmap(rpc_blob_t * __restrict blob);
/*
 * rpc_srv_blobFree() - Free blob from disk & memory
 *
 * @srv = RPC Server instance
 * @blob = Mapped BLOB element
 * return: -1 error or 0 ok
 */
int rpc_srv_blobFree(rpc_srv_t * __restrict srv, rpc_blob_t * __restrict blob);

/*
 * rpc_srv_registerBLOB() - Register new BLOB to server
 *
 * @srv = RPC Server instance
 * @len = BLOB length
 * @tout = BLOB live timeout in seconds
 * return: NULL error or new registered BLOB
 */
rpc_blob_t *rpc_srv_registerBLOB(rpc_srv_t * __restrict srv, size_t len, int tout);
/*
 * rpc_srv_unregisterBLOB() - Unregister BLOB from server
 *
 * @srv = RPC Server instance
 * @var = BLOB Variable for unregister
 * return: -1 error, 0 not found call, 1 unregister ok
 */
int rpc_srv_unregisterBLOB(rpc_srv_t * __restrict srv, uint32_t var);
/*
 * rpc_srv_getBLOB() - Get registered BLOB 
 *
 * @srv = RPC Server instance
 * @var = hash for variable
 * return: NULL not found, !=NULL return blob var
 */
rpc_blob_t *rpc_srv_getBLOB(rpc_srv_t * __restrict srv, uint32_t var);

/*
 * rpc_srv_sendBLOB() - Send mapped BLOB to client
 *
 * @cli = Client instance
 * @blob = Mapped BLOB element
 * return: -1 error, 0 ok
 */
int rpc_srv_sendBLOB(rpc_cli_t * __restrict cli, rpc_blob_t * __restrict blob);
/*
 * rpc_srv_recvBLOB() - Receive BLOB from client
 *
 * @cli = Client instance
 * @blob = Mapped BLOB element
 * return: -1 error, 0 ok, >0 unreceived data from client, may be error?
 */
int rpc_srv_recvBLOB(rpc_cli_t * __restrict cli, rpc_blob_t * __restrict blob);

/* CLIENT part of functions */

/*
 * rpc_cli_sendBLOB() - Send BLOB to server
 *
 * @cli = Client instance
 * @var = BLOB variable
 * @data = BLOB data
 * @tout = BLOB live on server timeout in seconds, if =0 default timeout
 * return: -1 error, 0 ok, 1 remote error
 */
int rpc_cli_sendBLOB(rpc_cli_t * __restrict cli, ait_val_t * __restrict var, 
		void * __restrict data, int tout);
/*
 * rpc_cli_recvBLOB() - Receive BLOB from server
 *
 * @cli = Client instance
 * @var = BLOB variable
 * @data = BLOB data, must be e_free after use!
 * return: -1 error, 0 ok, 1 remote error
 */
int rpc_cli_recvBLOB(rpc_cli_t * __restrict cli, ait_val_t * __restrict var, void ** __restrict data);
/*
 * rpc_cli_delBLOB() - Delete BLOB from server
 *
 * @cli = Client instance
 * @var = BLOB variable
 * return: -1 error, 0 ok, 1 remote error
 */
int rpc_cli_delBLOB(rpc_cli_t * __restrict cli, ait_val_t * __restrict var);
/*
 * rpc_cli_getBLOB() - Receive BLOB from server and Delete after that.
 *
 * @cli = Client instance
 * @var = BLOB variable
 * @data = BLOB data, must be e_free after use!
 * return: -1 error, 0 ok, >0 remote error
 */
int rpc_cli_getBLOB(rpc_cli_t * __restrict cli, ait_val_t * __restrict var, 
		void ** __restrict data);


/* RPC Client side functions */

/*
 * rpc_cli_openClient() - Connect to RPC Server
 *
 * @InstID = InstID for RPC session request
 * @netBuf = Network buffer length (min:512 bytes), if =0 == BUFSIZ (also meaning max RPC packet)
 * @csHost = Host name or IP address for bind server
 * @Port = Port for bind server, if Port == 0 default port is selected
 * @proto = Protocol, if == 0 choose SOCK_STREAM
 * return: NULL == error or !=NULL connection to RPC server established
 */
rpc_cli_t *rpc_cli_openClient(unsigned char InstID, int netBuf, 
		const char *csHost, unsigned short Port, int proto);
/*
 * rpc_cli_reconnectClient() - Reconnecting client to RPC server
 *
 * @cli = RPC Client session
 * return: -1 error or 0 ok
 */
int rpc_cli_reconnectClient(rpc_cli_t * __restrict cli);
/*
 * rpc_cli_closeClient() - Close connection to RPC server and free resources
 *
 * @cli = RPC Client session
 * return: none
 */
void rpc_cli_closeClient(rpc_cli_t ** __restrict cli);
/*
 * rpc_pkt_Send() - Send RPC packet
 *
 * @sock = Socket
 * @type = Type of socket
 * @sa = Server address
 * @pkt = RPC packet
 * @len = Length of packet
 * return: -1 error, 0  EOF or >0 sended bytes
 */
int rpc_pkt_Send(int sock, int type, sockaddr_t * __restrict sa, 
		ait_val_t * __restrict pkt, int len);
/*
 * rpc_pkt_Receive() - Receive RPC packet
 *
 * @sock = Socket
 * @type = Type of socket
 * @sa = Server address
 * @pkt = RPC packet
 * return: -1 error, 0 EOF or >0 received bytes
 */
int rpc_pkt_Receive(int sock, int type, sockaddr_t * __restrict sa, 
		ait_val_t * __restrict pkt);
/*
 * rpc_pkt_Request() - Build RPC Request packet
 *
 * @pkt = Packet buffer
 * @sess = RPC session info
 * @tag = Function tag for execution
 * @vars = Function argument array of values, may be NULL
 * @noreply = We not want RPC reply
 * @nocrc = Without CRC calculation
 * return: -1 error or != -1 prepared bytes into packet
 */
int rpc_pkt_Request(ait_val_t * __restrict pkt, rpc_sess_t * __restrict sess, 
		unsigned short tag, array_t * __restrict vars, int noreply, int nocrc);
/*
 * rpc_pkt_Replay() - Decode RPC Replay packet
 *
 * @pkt = Packet buffer
 * @sess = RPC session info
 * @tag = Function tag
 * @vars = Function argument array of values, may be NULL
 * @nocrc = Without CRC calculation
 * return: -1 error or != -1 return value from function
 */
int rpc_pkt_Replay(ait_val_t * __restrict pkt, rpc_sess_t * __restrict sess, 
		unsigned short tag, array_t ** __restrict vars, int nocrc);
/*
 * rpc_cli_execCall() - Execute RPC call
 *
 * @cli = RPC Client session
 * @noreply = We not want RPC reply
 * @tag = Function tag for execution
 * @in_vars = IN function argument array of values, may be NULL
 * @out_vars = OUT returned array of rpc values, if !=NULL must be free after use with ait_freeVars()
 * return: -1 error, 0 ok result or 1 closed rpc connection
 */
int rpc_cli_execCall(rpc_cli_t *cli, int noreply, unsigned short tag, 
		array_t * __restrict in_vars, array_t ** __restrict out_vars);
/*
 * rpc_cli_freeCall() - Free resouce allocated by RPC call
 *
 * @out_vars = Returned array with variables from RPC call
 * return: none
 */
void rpc_cli_freeCall(array_t ** __restrict out_vars);
/*
 * rpc_cli_ping() - Ping RPC server
 *
 * @cli = connected client
 * return: -1 error or !=-1 ping seq id
 */
int rpc_cli_ping(rpc_cli_t *cli);


/*
 * rpc_cli_openBLOBClient() - Connect to BLOB Server
 *
 * @rpccli = RPC Client session
 * @Port = Port for bind server, if Port == 0 default port is selected
 * return: NULL == error or !=NULL connection to BLOB server established
 */
rpc_cli_t *rpc_cli_openBLOBClient(rpc_cli_t * __restrict rpccli, unsigned short Port);
/*
 * rpc_cli_closeBLOBClient() - Close connection to BLOB server and free resources
 *
 * @cli = BLOB Client session
 * return: none
 */
void rpc_cli_closeBLOBClient(rpc_cli_t ** __restrict cli);


#endif
